@@ -23,14 +23,14 @@
 **
 ****************************************************************************/
 
-char *CM_DESC[] = { "COPY", "MOVE", "LINK" };
+const char *CM_DESC[] = { "COPY", "MOVE", "LINK" };
 char *copy_buff = NULL;
 
 int ignore_copy_errors = 0; /* actually it is used for copy/move/erase */
 
 /* clipboard ***************************************************************/
 
-char *CB_DESC[] = { "COPY", "MOVE", "SYMLINK" };
+const char *CB_DESC[] = { "COPY", "MOVE", "SYMLINK" };
 
 VTrie Clipboard;
 CopyInfo clipboard_copy_info;
@@ -51,7 +51,7 @@ fsize_t device_free_space( const char *target ) /* user free space, NOT real! */
 /*
  *  return 0 if src and dst are actually the same file
  */
-int file_is_same( const char *src, const char *dst ) 
+int file_is_same( const char *src, const char *dst )
 {
   #ifdef _TARGET_GO32_
     char _f1[MAX_PATH];
@@ -63,7 +63,7 @@ int file_is_same( const char *src, const char *dst )
   #else
     struct stat st1;
     struct stat st2;
-    if(stat( src, &st1 )) return 1; 
+    if(stat( src, &st1 )) return 1;
     if(stat( dst, &st2 )) return 1;
     return !( st1.st_dev == st2.st_dev && /* same device */
               st1.st_ino == st2.st_ino ); /* same inode */
@@ -73,7 +73,7 @@ int file_is_same( const char *src, const char *dst )
 /*
   return 0 if src and dst are on the same device
 */
-int device_is_same( const char *src, const char *dst ) 
+int device_is_same( const char *src, const char *dst )
 {
   #ifdef _TARGET_GO32_
     char _f1[MAX_PATH];
@@ -89,11 +89,11 @@ int device_is_same( const char *src, const char *dst )
     char _f1[MAX_PATH];
     char _f2[MAX_PATH];
     strcpy( _f1, src );
-    ch = strrchr( _f1, '/' ); 
+    ch = strrchr( _f1, '/' );
     if (ch == NULL) _f1[0] = 0; else ch[1] = 0;
     strcat( _f1, "." );
     strcpy( _f2, dst );
-    ch = strrchr( _f2, '/' ); 
+    ch = strrchr( _f2, '/' );
     if (ch == NULL) _f2[0] = 0; else ch[1] = 0;
     strcat( _f2, "." );
     if(stat( _f1, &st1 )) return 1;
@@ -105,9 +105,9 @@ int device_is_same( const char *src, const char *dst )
 #ifdef _TARGET_GO32_
   int fast_stat( const char* s, struct stat *st )
   {
-      /* 
+      /*
         NOTE: vfu does not use this info, so don't simulate it
-        under DOS/WinXX -- otherwise it is too slow 
+        under DOS/WinXX -- otherwise it is too slow
       */
       _djstat_flags =  _STAT_INODE       /* don't simulate inode info */
                       |_STAT_EXEC_EXT    /* don't try recognize exe's */
@@ -130,14 +130,14 @@ void show_copy_pos( fsize_t a_fc, /* file under copy current pos */
                     CopyInfo *copy_info ) /* totals info */
 {
   char t[16];
-  
+
   fsize_t c1 = a_fc;
   fsize_t a1 = a_fa;
   fsize_t c2 = copy_info->current_size;
   fsize_t a2 = copy_info->files_size;
 
   ASSERT( a1 >= 0 && a2 >= 0 );
-  
+
   if ( a1 < 1 ) a1 = 1;
   if ( a2 < 1 ) a2 = 1;
   if ( c1 == a1 ) /* hack, every single 100% each is not meaningfull really */
@@ -160,12 +160,12 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
   if ( access( dst, F_OK) ) return 1; /* doesn't exist -- copy... */
   if ( copy_info->over_mode == OM_NEVER  ) return 0; /* skip it */
   if ( copy_info->over_mode == OM_ALWAYS ) return 1; /* overwrite! */
-  
+
   struct stat stat_src;
   struct stat stat_dst;
   fast_stat( src, &stat_src );
   fast_stat( dst, &stat_dst );
-  
+
   if ( copy_info->over_mode == OM_ALWAYS_IF_MTIME &&
        stat_src.st_mtime > stat_dst.st_mtime ) return 1; /* newer mtime, do it! */
 
@@ -176,20 +176,20 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
     vfu_redraw_status();
     VString str;
     char sttime[32];
-  
+
     char s_t = (stat_src.st_mtime == stat_dst.st_mtime)?'*':' '; // same time
     char s_s = (stat_src.st_size  == stat_dst.st_size)?'*':' '; // same size
-  
+
     char t[MAX_PATH];
-  
-    time_str_compact( stat_src.st_mtime, sttime); 
-    str = file_st_size( &stat_src ); 
+
+    time_str_compact( stat_src.st_mtime, sttime);
+    str = file_st_size( &stat_src );
     str_comma(str);
     sprintf(t, "SRC: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, src );
     say1(t);
-  
-    time_str_compact(stat_dst.st_mtime, sttime); 
-    str = file_st_size( &stat_dst ); 
+
+    time_str_compact(stat_dst.st_mtime, sttime);
+    str = file_st_size( &stat_dst );
     str_comma(str);
     sprintf(t, "DST: %s%c %11s%c %s", sttime, s_t, str.data(), s_s, dst );
     say2(t);
@@ -201,7 +201,7 @@ int over_if_exist( const char* src, const char *dst, CopyInfo* copy_info )
       {
       VString diff = vfu_temp();
       VString cmd;
-      cmd = shell_diff + " '" + src + "' '" + dst + "' > " + diff;
+      cmd = shell_diff + " '" + dst + "' '" + src + "' > " + diff;
       system( cmd );
       vfu_browse( diff );
       unlink( diff );
@@ -231,7 +231,7 @@ int vfu_copy_mode( const char* src, const char* dst )
 {
   struct stat st;
   if (stat( src, &st )) return 1; /* FIXME: or silent? */
-  
+
   /* copy mode */
   mode_str_t mode_str;
   file_get_mode_str( st.st_mode, mode_str );
@@ -246,8 +246,8 @@ int vfu_copy_mode( const char* src, const char* dst )
   /* copy owner/group */
   if (opt.copy_keep_mode)
     chown( dst, st.st_uid, st.st_gid );
-    
-  return 0;  
+
+  return 0;
 };
 
 /***************************************************************************
@@ -267,13 +267,13 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
 
   fsize_t size = file_size( src );
   if ( size == -1 ) return 1;
-  
+
   if (!over_if_exist( src, dst, copy_info ))
     {
     copy_info->current_size += size; /* consider it ok */
     return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
     }
-  
+
   if ( file_exist( dst ) )
     { /* destination file exists */
     if ( file_is_same( src, dst ) == 0 )
@@ -287,7 +287,7 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
   str = "COPY TO: " + str;
   say1( str );
   con_out( 1, con_max_y(), copy_info->description, cMESSAGE );
-  
+
 
   if ( !copy_info->no_free_check && !copy_info->no_info )
     {
@@ -297,10 +297,10 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
       char t[128];
       vfu_beep();
       sprintf(t, "Insufficient disk space! Free: %.0f, Required: %.0f",
-                  dev_free, size ); 
+                  dev_free, size );
       say1( t );
       say2( dst );
-      
+
       vfu_menu_box( "Error prompt", "C Continue anyway,S Skip file,N No free space check,  Abort (ESC)", -1 );
       switch (menu_box_info.ec)
         {
@@ -342,12 +342,12 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
       }
     z = fread(copy_buff, 1, COPY_BUFFER_SIZE, f1);
     if (z > 0) z = fwrite(copy_buff, 1, z, f2);
-    if (z == -1) 
-      { 
-      fclose(f1); 
+    if (z == -1)
+      {
+      fclose(f1);
       fclose(f2);
       unlink( dst ); /* remove dst if partially copied */
-      return 1; 
+      return 1;
       }
     cp += z;
     ASSERT( cp <= size );
@@ -378,17 +378,17 @@ int __vfu_file_copy( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_file_move( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
-  
+
   if ( device_is_same( src, dst ) == 0 )
     { /* same device */
-    if (!over_if_exist( src, dst, copy_info )) 
+    if (!over_if_exist( src, dst, copy_info ))
       return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
-      
+
     if ( file_exist( dst ) )
       {
       if ( file_is_same( src, dst ) == 0 )
         return CR_SKIP; /* dst is src actually */
-      /* FIXME: what if dst is symlink? */  
+      /* FIXME: what if dst is symlink? */
       if ( __vfu_file_erase( dst ) ) return 1;
       }
     if ( rename( src, dst ) ) return 1;
@@ -412,23 +412,23 @@ int __vfu_file_move( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
-  
+
   VString fname_src; /* used for directory entries */
   VString fname_dst; /* used for directory entries */
 
-  if ( vfu_break_op() ) 
+  if ( vfu_break_op() )
     return CR_ABORT; /* canceled */
-  
+
   if ( file_is_same( src, dst ) == 0 )
     return CR_SKIP;
-  
-  if ( make_path( dst ) ) 
+
+  if ( make_path( dst ) )
     {
     if ( ignore_copy_errors ) return CR_ABORT; /* cancel operation */
-    
+
     say1( dst );
     say2errno();
-    vfu_menu_box( "Create dir error", 
+    vfu_menu_box( "Create dir error",
                   "C Continue anyway,I Ignore further errors,  Abort (ESC)", -1 );
     if ( menu_box_info.ec != 'C' )
       return CR_ABORT; /* cancel operation */
@@ -438,18 +438,18 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
       return CR_ABORT; /* cancel operation */
       }
     }
-    
+
   DIR *dir;
   dirent *de;
-  
+
   dir = opendir( src );
   if (!dir) return 1; /* FIXME: report error? */
-  
+
   while( (de = readdir(dir)) )
     {
     if (strcmp( de->d_name, ".") == 0) continue;
     if (strcmp( de->d_name, "..") == 0) continue;
-    
+
     fname_src  = src;
     fname_src += "/";
     fname_src += de->d_name;
@@ -457,7 +457,7 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
     fname_dst  = dst;
     fname_dst += "/";
     fname_dst += de->d_name;
-    
+
     while(4)
       {
       int r = __vfu_copy( fname_src, fname_dst, copy_info );
@@ -470,10 +470,10 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
       if ( r && r != CR_SKIP )
         {
         if ( ignore_copy_errors ) break;
-        
+
         say1( fname_dst );
         say2errno();
-        vfu_menu_box( "Copy/Move/SymLink error", 
+        vfu_menu_box( "Copy/Move/SymLink error",
                       "T Try again,S Skip/continue,I Ignore further errors,  Abort (ESC)" );
         if ( menu_box_info.ec == 'T' )
           continue; /* while(4) */
@@ -484,21 +484,21 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
           {
           ignore_copy_errors = 1;
           break;
-          }  
+          }
         else
           {
           closedir(dir);
           return CR_ABORT;
-          }  
+          }
         }
       else
         break; /* copy (r) is ok -- exit error (while(4)) loop */
-      } /* while(4) */  
+      } /* while(4) */
     } /* readdir() */
-  closedir( dir );  
-  
+  closedir( dir );
+
   if ( vfu_copy_mode( src, dst ) ) return 1;
-  
+
   return 0;
 };
 
@@ -507,10 +507,10 @@ int __vfu_dir_copy( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_dir_move( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
-  
+
   if ( file_is_same( src, dst ) == 0 )
     return CR_SKIP;
-  
+
   if ( device_is_same( src, dst ) == 0 )
     { /* same device */
     if ( rename( src, dst ) ) return 1;
@@ -542,14 +542,14 @@ int __vfu_link_copy( const char* src, const char* dst, CopyInfo* copy_info )
 
     if (!over_if_exist( src, dst, copy_info ))
       return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
-    
+
     if ( file_exist( dst ) )
       { /* destination file exists */
       if ( file_is_same( src, dst ) == 0 )
         return CR_SKIP; /* dst is src actually */
       __vfu_file_erase( dst ); /* overwrite! */
       }
-    
+
     char t[MAX_PATH];
     int z = readlink( src, t, sizeof(t)-1);
     if (z < 1) return 1;
@@ -580,29 +580,29 @@ int __vfu_link_move( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
 {
   errno = 0; /* clear error status */
-  
+
   VString fname; /* used for directory entries */
   VString s;
-  
-  if ( vfu_break_op() ) 
+
+  if ( vfu_break_op() )
     return CR_ABORT; /* canceled */
-  
+
   if (!file_exist( target )) return 1;
-  
+
   /* make it writeable so we can erase files in it */
   file_set_mode_str( target, MODE_WRITE_ON );
-  
+
   DIR *dir;
   dirent *de;
-  
+
   dir = opendir( target );
   if (!dir) return 1; /* FIXME: report error? */
-  
+
   while( (de = readdir(dir)) )
     {
     if (strcmp( de->d_name, ".") == 0) continue;
     if (strcmp( de->d_name, "..") == 0) continue;
-    
+
     fname = target;
     fname += "/";
     fname += de->d_name;
@@ -611,9 +611,9 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
       {
       /* progress report */
       say1( fname );
-      
+
       int r = __vfu_erase( fname, bytes_freed );
-      
+
       if ( r == CR_ABORT )
         {
         closedir(dir);
@@ -622,10 +622,10 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
       if ( r && r != CR_SKIP )
         {
         if ( ignore_copy_errors ) break;
-        
+
         say1( fname );
         say2errno();
-        vfu_menu_box( "Erase error", 
+        vfu_menu_box( "Erase error",
                       "T Try again,S Skip/continue,I Ignore further errors,  Abort (ESC)" );
         if ( menu_box_info.ec == 'T' )
           continue; /* while(4) */
@@ -641,13 +641,13 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
           {
           closedir(dir);
           return CR_ABORT;
-          }  
+          }
         }
       else
-        break;  
-      } /* while(4) */  
+        break;
+      } /* while(4) */
     } /* readdir() */
-  closedir( dir );  
+  closedir( dir );
 
   /* show bytes freed if required */
   if ( bytes_freed )
@@ -667,9 +667,9 @@ int __vfu_dir_erase( const char* target, fsize_t* bytes_freed )
 int __vfu_file_erase( const char* target, fsize_t* bytes_freed  )
 {
   errno = 0; /* clear error status */
-  
+
   #ifdef _TARGET_GO32_
-  /* under dos write access rules and delete protection 
+  /* under dos write access rules and delete protection
      so we have to remove it first */
   file_set_mode_str( target, MODE_WRITE_ON );
   #endif
@@ -678,7 +678,7 @@ int __vfu_file_erase( const char* target, fsize_t* bytes_freed  )
     target_size = file_size( target );
 
   int r = unlink( target );
-  
+
   if ( r == 0 && bytes_freed )
     *bytes_freed += target_size;
   return (r != 0);
@@ -689,7 +689,7 @@ int __vfu_file_erase( const char* target, fsize_t* bytes_freed  )
 int __vfu_link_erase( const char* target, fsize_t* bytes_freed )
 {
   errno = 0; /* clear error status */
-  
+
   #ifdef _TARGET_GO32_
     return 0; /* always ok under dos */
   #else
@@ -706,12 +706,12 @@ int __vfu_copy( const char* src, const char* dst, CopyInfo* copy_info )
   if ( file_is_link( src ) )
     r = __vfu_link_copy( src, dst, copy_info ); /* symlink */
   else
-  if ( file_is_dir( src ) )  
+  if ( file_is_dir( src ) )
     r = __vfu_dir_copy( src, dst, copy_info ); /* directory */
   else
     r = __vfu_file_copy( src, dst, copy_info ); /* regular file */
-  
-  return r;  
+
+  return r;
 };
 
 /*---------------------------------------------------------------------------*/
@@ -722,12 +722,12 @@ int __vfu_move( const char* src, const char* dst, CopyInfo* copy_info )
   if ( file_is_link( src ) )
     r = __vfu_link_move( src, dst, copy_info ); /* symlink */
   else
-  if ( file_is_dir( src ) )  
+  if ( file_is_dir( src ) )
     r = __vfu_dir_move( src, dst, copy_info ); /* directory */
   else
     r = __vfu_file_move( src, dst, copy_info ); /* regular file */
-  
-  return r;  
+
+  return r;
 };
 
 /*---------------------------------------------------------------------------*/
@@ -735,15 +735,15 @@ int __vfu_move( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_symlink( const char* src, const char* dst, CopyInfo* copy_info )
 {
   errno = 0; /* clear error status */
-  
-  if (!over_if_exist( src, dst, copy_info )) 
+
+  if (!over_if_exist( src, dst, copy_info ))
     return copy_info->abort ? CR_ABORT : CR_SKIP; /* ok */
-  
+
   if ( file_exist( dst ) )
     {
     if ( file_is_same( src, dst ) == 0 )
       return CR_SKIP; /* dst is src actually */
-    /* FIXME: what if dst is symlink? */  
+    /* FIXME: what if dst is symlink? */
     if ( __vfu_file_erase( dst ) ) return 1;
     }
   int r = symlink( src, dst );
@@ -755,16 +755,16 @@ int __vfu_symlink( const char* src, const char* dst, CopyInfo* copy_info )
 int __vfu_erase( const char* target, fsize_t* bytes_freed )
 {
   int r = 0;
-  
+
   if ( file_is_link( target ) )
     r = __vfu_link_erase( target, bytes_freed ); /* symlink */
   else
-  if ( file_is_dir( target ) )  
+  if ( file_is_dir( target ) )
     r = __vfu_dir_erase( target, bytes_freed ); /* directory */
   else
     r = __vfu_file_erase( target, bytes_freed ); /* regular file */
-  
-  return r;  
+
+  return r;
 };
 
 /* high-level interface functions ******************************************/
@@ -792,9 +792,9 @@ void vfu_copy_files( int a_one, int a_mode )
     say1( "No files" );
     return;
     }
-  
+
   char t[MAX_PATH];
-  
+
   ASSERT( a_mode == CM_COPY || a_mode == CM_MOVE || a_mode == CM_LINK );
 
   #ifdef _TARGET_GO32_
@@ -804,13 +804,17 @@ void vfu_copy_files( int a_one, int a_mode )
     return;
     }
   #endif
-  
+
   CopyInfo copy_info;
 
   if ( opt.copy_calc_totals == 2 ) /* PRELIMINARY copy calc totals */
     __copy_calc_totals( copy_info, a_one );
 
-  VString target = opt.last_copy_path[ a_mode ];
+  VString target;
+  if( opt.default_copy_to_cwd )
+    target = work_path;
+  else
+    target = opt.last_copy_path[ a_mode ];
   const char* cm_mode_str[] = { "COPY", "MOVE", "LINK" };
   VString str = cm_mode_str[ a_mode ];
   if ( a_one )
@@ -826,8 +830,8 @@ void vfu_copy_files( int a_one, int a_mode )
     {
     say1( "Target is file, not directory!" );
     say2( t );
-    
-    vfu_menu_box( "Error prompt", 
+
+    vfu_menu_box( "Error prompt",
                   "C Continue anyway,E Erase first,  Abort (ESC)", -1 );
     if ( menu_box_info.ec == 'E' )
       {
@@ -842,7 +846,7 @@ void vfu_copy_files( int a_one, int a_mode )
 
   if ( opt.copy_calc_totals == 1 ) /* copy calc totals if not PRELIMINARY */
     __copy_calc_totals( copy_info, a_one );
-  
+
   copy_info.no_free_check = !opt.copy_free_space_check;
   copy_info.over_mode = OM_ASK; /* 0 */
   copy_info.abort = 0;
@@ -854,11 +858,11 @@ void vfu_copy_files( int a_one, int a_mode )
       {
       vfu_beep();
       sprintf(t, "Insufficient disk space! Free: %.0f, Required: %.0f",
-                 dev_free, copy_info.files_size ); 
+                 dev_free, copy_info.files_size );
       say1( t );
       say2( target );
-      
-      vfu_menu_box( "Error prompt", 
+
+      vfu_menu_box( "Error prompt",
                     "C Continue anyway,  Abort (ESC)", -1 );
       if ( menu_box_info.ec != 'C' ) return; /* abort */
       }
@@ -871,7 +875,7 @@ void vfu_copy_files( int a_one, int a_mode )
   str_comma( t );
   copy_info.description += t;
   copy_info.description += " bytes.";
-    
+
   ASSERT( !copy_buff );
   copy_buff = new char[1024*1024];
   ASSERT( copy_buff );
@@ -880,7 +884,7 @@ void vfu_copy_files( int a_one, int a_mode )
     say1( "Copy error: cannot allocate copy buffer" );
     return;
     }
-  
+
   int z;
   for ( z = 0; z < files_count; z++ )
     {
@@ -898,19 +902,19 @@ void vfu_copy_files( int a_one, int a_mode )
     VString src  = fi->full_name();
     VString dst  = target;
            dst += fi->name_ext();
-    
+
     int r = 0;
     if ( a_mode == CM_COPY ) r = __vfu_copy( src, dst, &copy_info ); else
     if ( a_mode == CM_MOVE ) r = __vfu_move( src, dst, &copy_info ); else
     if ( a_mode == CM_LINK ) r = __vfu_symlink( src, dst, &copy_info ); else
     ASSERT(!"Bad copy mode");
-    
+
     if ( r == 0 )
       {
       if ( a_mode == CM_MOVE )
-        { 
-        delete fi; 
-        files_list[z] = NULL; 
+        {
+        delete fi;
+        files_list[z] = NULL;
         }
       }
     else if ( r != CR_SKIP && r != CR_ABORT )
@@ -927,12 +931,12 @@ void vfu_copy_files( int a_one, int a_mode )
   if ( a_mode == CM_MOVE )
     vfu_pack_files_list();
   update_status();
-  do_draw = 2;     
-     
+  do_draw = 2;
+
   ASSERT( copy_buff );
   delete [] copy_buff;
   copy_buff = NULL;
-  
+
   say1( "" );
   /* show bytes copied */
   if ( copy_info.current_size > 0 )
@@ -961,7 +965,7 @@ void vfu_erase_files( int a_one )
     say1( "No files" );
     return;
     }
-  
+
   fsize_t bytes_freed = 0;
   fsize_t *bytes_freed_ptr = opt.bytes_freed ? &bytes_freed : NULL;
 
@@ -990,27 +994,27 @@ void vfu_erase_files( int a_one )
   say1("");
   say2("");
   if (ch != 13) return;
-  
+
   int z;
   for ( z = 0; z < files_count; z++ )
     {
     if ( vfu_break_op() ) break; /* cancel operation */
     TF *fi = files_list[z];
-    
-    if ( a_one && z != FLI ) continue; 
+
+    if ( a_one && z != FLI ) continue;
     if ( !a_one && !fi->sel ) continue;
-    
+
     VString target  = fi->full_name();
-           
+
     int r = __vfu_erase( target, bytes_freed_ptr );
-      
+
     if ( r == 0 )
       { delete fi; files_list[z] = NULL; }
     else if ( r != CR_ABORT && !ignore_copy_errors )
       {
       say1( target );
       say2errno();
-      vfu_menu_box( "Erase error", 
+      vfu_menu_box( "Erase error",
                     "C Continue operation,I Ignore further errors,  Abort (ESC)" );
       if ( menu_box_info.ec != 'C' && menu_box_info.ec != 'I' )
         r = CR_ABORT;
@@ -1019,11 +1023,11 @@ void vfu_erase_files( int a_one )
       }
     if ( r == CR_ABORT ) break; /* cancel operation */
     } /* for files_list[] */
-  
+
   vfu_pack_files_list();
   update_status();
-  do_draw = 2;     
-  
+  do_draw = 2;
+
   say1("");
   /* show bytes freed if required */
   if ( opt.bytes_freed )
@@ -1035,7 +1039,7 @@ void vfu_erase_files( int a_one )
     }
   else
     say2( "ERASE DONE" );
-  
+
   ignore_copy_errors = 0;
 };
 
@@ -1053,12 +1057,12 @@ void clipboard_add()
                      clipboard_copy_info.files_count );
     return;
     }
-  
+
   Clipboard.undef();
   clipboard_copy_info.reset();
-  
+
   VString keep = "1";
-  
+
   int z;
   for ( z = 0; z < files_count; z++ )
     {
@@ -1068,22 +1072,22 @@ void clipboard_add()
     }
 
   __copy_calc_totals( clipboard_copy_info, 0 );
-  
+
   clipboard_copy_info.no_free_check = !opt.copy_free_space_check;
   clipboard_copy_info.over_mode = OM_ASK; /* 0 */
   clipboard_copy_info.abort = 0;
-  
-  say( 1, cINFO, "CLIPBOARD: %d files added.", 
+
+  say( 1, cINFO, "CLIPBOARD: %d files added.",
                  clipboard_copy_info.files_count );
-  say2( "" );               
+  say2( "" );
 };
 
 void clipboard_paste( int mode )
 {
   VArray va = Clipboard.keys();
 
-  ASSERT(    mode == CLIPBOARD_COPY 
-          || mode == CLIPBOARD_MOVE 
+  ASSERT(    mode == CLIPBOARD_COPY
+          || mode == CLIPBOARD_MOVE
           || mode == CLIPBOARD_SYMLINK );
 
   ASSERT( !copy_buff );
@@ -1094,13 +1098,13 @@ void clipboard_paste( int mode )
     say1( "Copy error: cannot allocate copy buffer" );
     return;
     }
-  
+
   va.reset();
   const char* ps;
   while( (ps = va.next()) )
     {
     VString dst = work_path + str_file_name_ext( ps );
-    
+
     int r;
     if ( mode == CLIPBOARD_SYMLINK )
       r = __vfu_symlink( ps, dst, &clipboard_copy_info );
@@ -1118,17 +1122,17 @@ void clipboard_paste( int mode )
       }
     if ( r == CR_ABORT ) break; // cancel operation
     }
-  
+
   ASSERT( copy_buff );
   delete [] copy_buff;
   copy_buff = NULL;
-  
+
   vfu_rescan_files( 0 );
-  say( 1, cINFO, "CLIPBOARD: %s: %d of %d files processed ok", 
+  say( 1, cINFO, "CLIPBOARD: %s: %d of %d files processed ok",
                  CB_DESC[ mode ],
-                 clipboard_copy_info.ok_count, 
+                 clipboard_copy_info.ok_count,
                  clipboard_copy_info.files_count );
-                 
+
   if ( mode == CLIPBOARD_MOVE ) clipboard_clear();
 };
 
@@ -1136,7 +1140,7 @@ void clipboard_clear()
 {
   if( clipboard_copy_info.files_count )
     say( 2, cINFO, "CLIPBOARD: %d files removed", clipboard_copy_info.files_count );
-  else  
+  else
     say( 2, cINFO, "CLIPBOARD: empty" );
   Clipboard.undef();
   clipboard_copy_info.reset();
@@ -1164,13 +1168,13 @@ void clipboard_menu( int act )
     mb.push( "L Symlink files here" );
     mb.push( "E Clear clipboard" );
     mb.push( "V View clipboard" );
-    
+
     VString fcnt = clipboard_copy_info.files_count;
     str_comma( fcnt );
     VString fsize = clipboard_copy_info.files_size;
     str_comma( fsize );
     mb.push( "---  " + fcnt + " files, " + fsize + " bytes" );
-    
+
     if ( vfu_menu_box( 5, 5, "File Clipboard " + fcnt + " files, " + fsize + " bytes" ) == -1 ) return;
     act = menu_box_info.ec;
     }
