@@ -5,7 +5,7 @@
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- * $Id: vfuopt.cpp,v 1.16 2004/07/08 01:25:52 cade Exp $
+ * $Id: vfuopt.cpp,v 1.18 2008/01/18 18:40:30 cade Exp $
  *
  */
 
@@ -18,13 +18,13 @@
 
 Options opt;
 
-char *NOYES[] = { " - ", "YES", NULL };
-char *NOYESPRECOPY[] = { " - ", "YES", "PRELIM", NULL };
-char *FTIMETYPE[] = { "CHANGE", "MODIFY", "ACCESS", NULL };
+const char *NOYES[] = { " - ", "YES", NULL };
+const char *NOYESPRECOPY[] = { " - ", "YES", "PRELIM", NULL };
+const char *FTIMETYPE[] = { "CHANGE", "MODIFY", "ACCESS", NULL };
 #ifdef _TARGET_GO32_
-char *TAGMARKS[] = { ">>", "=>", "->", "", "", " ", "", NULL };
+const char *TAGMARKS[] = { ">>", "=>", "->", "", "", " ", "", NULL };
 #else
-char *TAGMARKS[] = { ">>", "=>", "->", NULL };
+const char *TAGMARKS[] = { ">>", "=>", "->", NULL };
 #endif
 
 ToggleEntry Toggles[] =
@@ -51,12 +51,12 @@ ToggleEntry Toggles[] =
   {  0 , "--trees/dirs-- " , NULL, NULL },
   { ' ', "Compact DirTree", &(opt.tree_compact), NOYES },
   { ' ', "CDTree (cdpath) ", &(opt.tree_cd), NOYES },
-  { ' ', "Fast dir size cache ", &(opt.fast_size_cache), NOYES },
   {  0 , "--troubleshooting--", NULL, NULL },
   { ' ', "Clear screen on shell", &(opt.shell_cls), NOYES },
   {  0 , "--compatibility--", NULL, NULL },
   { ' ', "Lynx style navigation",       &(opt.lynx_navigation), NOYES },
   { ' ', "Mask auto expand", &(opt.mask_auto_expand), NOYES },
+  { ' ', "Use CWD as target for COPY/MOVE", &(opt.default_copy_to_cwd), NOYES },
   {  0 , "--other--", NULL, NULL },
 /*  { ' ', "Can Zap/Erase READ-ONLY Files?!",       &(opt.zap_ro), NOYES }, ? */
   { 'b', "Allow beep!",       &(opt.allow_beep), NOYES },
@@ -139,7 +139,7 @@ void vfu_load_dir_colors()
     str_replace( str, "EXEC", ".**" );
 
     str_ins( ext_colors[pos], 0, str );
-    
+
     };
 
   for ( int z = 0; z < 16; z++ )
@@ -149,7 +149,7 @@ void vfu_load_dir_colors()
       if ( opt.lower_case_ext_config )
         str_low( ext_colors[z] );
       }
-    
+
   #endif /* _TARGET_UNIX_ */
 };
 
@@ -210,7 +210,7 @@ int key_by_name( const char* key_name )
   if (strcmp (key_name, "MENU"  ) == 0) ux.key = - menucount;
   */
   VRegexp reFKEYS( "[\\@\\^\\#]?[fF][01234567890]+" );
-  
+
   if ( reFKEYS.m( key_name ) )
     {
     if ( toupper(key_name[0]) == 'F' )
@@ -222,7 +222,7 @@ int key_by_name( const char* key_name )
     if ( toupper(key_name[0]) == '#' )
       return KEY_SH_F1 + atoi( key_name + 2 ) - 1;
     }
-  return 0;    
+  return 0;
 };
 
 /*---------------------------------------------------------------------------*/
@@ -230,23 +230,23 @@ int key_by_name( const char* key_name )
 void vfu_settings_load()
 {
   VString str;
-  
+
   user_externals.undef();
   history.undef();
   see_filters.undef();
   panelizers.undef();
   archive_extensions.undef();
-  
+
   /***** LOAD DEFAULTS *******/
 
   memset( &opt, 0, sizeof( opt ) );
-  
+
   opt.svo.reset();
   opt.seo.reset();
   opt.seo.handle_tab = 1;
-  
-  opt.sort_order = 'N'; 
-  opt.sort_direction = 'A'; 
+
+  opt.sort_order = 'N';
+  opt.sort_direction = 'A';
   opt.sort_top_dirs = 1;
 
   opt.f_size = 1;
@@ -260,10 +260,9 @@ void vfu_settings_load()
   opt.long_name_view = 0;
   opt.tree_compact = 0;
   opt.tree_cd = 1;
-  opt.fast_size_cache = 0;
 
   opt.show_hidden_files = 1;
-  
+
   opt.allow_beep = 1;
 
   opt.use_colors = 1;
@@ -295,18 +294,18 @@ void vfu_settings_load()
   opt.bytes_freed = 1;
 
   /***** LOAD DEFAULTS END ***/
-  
+
   FILE *fsett;
-  
+
   Options tmp_opt;
   memset( &tmp_opt, 0, sizeof( tmp_opt ) );
   if ( file_load_crc32( filename_opt, &tmp_opt, sizeof( tmp_opt ) ) == 0 )
     memcpy( &opt, &tmp_opt, sizeof(Options) );
   else
     say1( "warning: bad vfu.options file, loading defaults..." );
-  
+
   history.fload( filename_history );
-  
+
   if (getenv("EDITOR"))
     {
     shell_editor  = getenv("EDITOR");
@@ -341,16 +340,16 @@ void vfu_settings_load()
       if ( line[0] == ';' ) continue;
       str_cut( line, "\n\r" );
       if ( strlen( line ) == 0 ) continue;
-      
+
       if(set_str( line, "browser", shell_browser))continue;
       if(set_str( line, "pager", shell_browser))continue;
       if(set_str( line, "viewer", shell_browser))continue;
 
       if(set_arr( line, "archive",  archive_extensions))continue;
-      
+
       if(set_str( line, "editor", shell_editor))continue;
       if(set_str( line, "diff",   shell_diff))continue;
-      
+
       if(set_arr( line, "bookmark",  path_bookmarks))continue;
       // follow 10 are deprecated
       if(set_arr( line, "bookmark1", path_bookmarks))continue;
@@ -362,7 +361,7 @@ void vfu_settings_load()
       if(set_arr( line, "bookmark7", path_bookmarks))continue;
       if(set_arr( line, "bookmark8", path_bookmarks))continue;
       if(set_arr( line, "bookmark9", path_bookmarks))continue;
-            
+
    /* if(set_str( line, "cblack"   , ext_colors[0]); */
       if(set_str( line, "cgreen"   , ext_colors[cGREEN]))continue;
       if(set_str( line, "cred"     , ext_colors[cRED]))continue;
@@ -379,9 +378,9 @@ void vfu_settings_load()
       if(set_str( line, "chmagenta", ext_colors[chMAGENTA]))continue;
       if(set_str( line, "chblue"   , ext_colors[chBLUE]))continue;
       if(set_str( line, "chyellow" , ext_colors[chYELLOW]))continue;
-    
+
       if(set_splitter( line, "trimtree",  trim_tree  ))continue;
-  
+
       /* following code is used to clean input data */
       if( re_ux.m( line ) )
         {
@@ -410,12 +409,12 @@ void vfu_settings_load()
       }
     fclose(fsett);
     }
-  
+
   #ifdef _TARGET_GO32_
   int z;
   for ( z = 0; z < 16; z++ ) str_low( ext_colors[z] );
   #endif
-  
+
   if (opt.use_dir_colors) vfu_load_dir_colors();
 //  if (file_load_crc32( filename_size_cache, &size_cache, sizeof(size_cache)))
 //    memset( &size_cache, 0, sizeof(size_cache) );
@@ -474,11 +473,11 @@ void vfu_options()
 {
   say1("press SPACE to toggle, ENTER or ESC to exit");
   say2("");
-  vfu_toggle_box( 30, 5, "Options/Toggles   (scroll down...)", Toggles );
+  vfu_toggle_box( 30, 5, "Options/Toggles (scroll down, SPACE selects)", Toggles );
   vfu_settings_save();
   vfu_settings_load();
   vfu_drop_all_views();
-  vfu_redraw(); 
+  vfu_redraw();
   vfu_redraw_status();
   say1("");
   say2("");
