@@ -5,7 +5,7 @@
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- * $Id: vfusetup.h,v 1.11 2005/06/05 22:00:10 cade Exp $
+ * $Id: vfusetup.h,v 1.13 2007/02/12 06:14:42 cade Exp $
  *
  */
 
@@ -20,10 +20,10 @@
  *
  */
 
-#define VFU_VERSION "4.06"
+#define VFU_VERSION "4.09"
 
-#define HEADER "VF/U v" VFU_VERSION " (" __DATE__ ") by (c) Vladi Belperchinov-Shabanski `Cade' 1996-2005"
-#define CONTACT "<cade@biscom.net> <cade@datamax.bg> [http://soul.datamax.bg/~cade]"
+#define HEADER "VF/U v" VFU_VERSION " (" __DATE__ ") by (c) Vladi Belperchinov-Shabanski `Cade' 1996-2009"
+#define CONTACT "<cade@biscom.net> <cade@datamax.bg> [http://cade.datamax.bg]"
 
 #ifdef _TARGET_GO32_
   #define FILENAME_OPT     "vfu.opt"
@@ -71,7 +71,7 @@
 #define cWARNING (CONCOLOR(chWHITE,cRED)) // warning messages
 
 #define cBAR     (CONCOLOR(chWHITE,cBLUE)) // inverted select bar (dir tree)
-#define cTAG     (CONCOLOR(cRED,cWHITE)) // currently selected file 
+#define cTAG     (CONCOLOR(cRED,cWHITE)) // currently selected file
 
 #define cMENU_CN (CONCOLOR(chWHITE,cBLUE)) // menu normal
 #define cMENU_CH (CONCOLOR(chWHITE,cGREEN)) // menu highlite
