@@ -5,7 +5,7 @@
  *
  * SEE `README',`LICENSE' OR `COPYING' FILE FOR LICENSE AND OTHER DETAILS!
  *
- * $Id: vfuuti.cpp,v 1.16 2005/06/05 22:00:10 cade Exp $
+ * $Id: vfuuti.cpp,v 1.19 2007/12/16 13:52:50 cade Exp $
  *
  */
 
@@ -24,6 +24,16 @@ fsize_t file_st_size( struct stat* st )
   return st->st_size + ( st->st_size < 0 ) * ((uintmax_t)TYPE_MAXIMUM(off_t) - TYPE_MINIMUM(off_t) + 1);
 }
 
+VString vfu_readlink( const char* fname )
+{
+  char t[MAX_PATH+1];
+  t[0] = 0;
+  int l = readlink( fname, t, MAX_PATH );
+  if (l != -1) t[l] = 0;
+  VString res = t;
+  return res;
+}
+
 /*---------------------------------------------------------------------------*/
 
 int vfu_update_shell_line( VString &a_line, VString &a_options )
@@ -32,6 +42,8 @@ VString out;
 VString s;
 
 int i = 0;
+int one = 0;
+int full = 0;
 
 // toggles
 #ifdef _TARGET_GO32_
@@ -46,44 +58,64 @@ while( a_line[i] )
       #ifdef _TARGET_GO32_
       case '_' : use_SFN = 1; break;
       #endif
-      
+
       case 'r' : /* rescan files after */
       case 'R' : a_options += "r"; break; // refresh all files after (readdir)
       case 'f' : /* file name */
-      case 'F' : if ( files_count > 0 && work_mode == WM_ARCHIVE )
+      case 'F' :
+      case 'g' :
+      case 'G' : one = 0;
+                 if ( a_line[i+1] == 'f' ) one = 1;
+                 if ( a_line[i+1] == 'F' ) one = 1;
+                 full = 0;
+                 if ( a_line[i+1] == 'F' ) full = 1;
+                 if ( a_line[i+1] == 'G' ) full = 1;
+                 // FIXME: 'one' is used to merge F and G options in the future
+                 int z;
+                 for( z = 0; z < files_count; z++ )
                    {
-                   /*
-                     If in WM_ARCHIVE, suppose we are in temp
-                     directory already and required file is extracted
-                     in here... so we don't use full_name()
-                   */
-                   out += files_list[FLI]->name();
-                   }
-                 else
-                 if ( files_count > 0 )  
-                   {
-                   s = ( a_line[i+1] == 'F' ) ?
-                       files_list[FLI]->full_name() 
-                       :
-                       files_list[FLI]->name(); 
-                   #ifdef _TARGET_GO32_
-                   if (use_SFN)
+                   TF *fi = files_list[z];
+                   if (  one && z != FLI ) continue; /* if one and not current -- skip */
+                   if ( !one && !fi->sel ) continue; /* if not one and not selected -- skip */
+                   if( work_mode == WM_ARCHIVE )
                      {
-                     char short_name[32]; /* well longest short dos name is 12 */
-                     file_get_sfn( s, short_name );
-                     s = short_name;
+                     s = files_list[z]->name();
+                     }
+                   else if( full )
+                     {
+                     files_list[z]->full_name();
+                     }
+                   else
+                     {
+                     s = files_list[z]->name();
+                     }
+                   if( !one )
+                     {
+                     // need to list files, quote them...
+                     // FIXME: this should be optional for all one and !one
+                     if ( str_find( s, "'" ) == -1 )
+                       {
+                       s = "'" + s +"' ";
+                       }
+                     else if ( str_find( s, "\"" ) == -1 )
+                       {
+                       s = "\"" + s +"\" ";
+                       }
+                     else
+                       {
+                       s = "";
+                       }
                      }
-                     str_tr( s, "/", "\\" ); /* use \ under dos */
-                   #endif /* _TARGET_GO32_ */
                    out += s;
                    }
                  break;
+
       case 'e' : /* name only */
       case 'E' : /* extension only */
                  if ( a_line[i+1] == 'e' )
                    s = str_file_name( s );
                  else
-                   s = files_list[FLI]->ext(); 
+                   s = files_list[FLI]->ext();
                  out += s;
                  break;
       case 's' : /* current file size */
@@ -110,7 +142,7 @@ while( a_line[i] )
                  #endif
                  out += s;
                  break;
-  
+
       case 'C' : /* startup dir */
                  s = startup_path;
                  #ifdef _TARGET_GO32_
@@ -118,7 +150,7 @@ while( a_line[i] )
                  #endif
                  out += s;
                  break;
-  
+
       case 'a' : /* Archive name */
                  out += archive_name;
                  break;
@@ -129,11 +161,12 @@ while( a_line[i] )
                  #endif
                  out += s;
                  break;
-  
+
       case 'w' :
       case 'W' : a_options += "w"; break;
       case 'i' : a_options += "i"; break;
       case 'n' : a_options += "n"; break;
+      case '!' : a_options += "!"; break;
       default  : /* chars not recognized are accepted "as is" */
                  str_add_ch( out, a_line[i+1] );
                  break;
@@ -172,6 +205,7 @@ fsize_t vfu_update_sel_size( int one ) // used before copy/move to calc estimate
 {
   fsize_t size = 0;
   int z;
+  int need_size_cache_sort = 0;
   for( z = 0; z < files_count; z++ )
     {
     TF *fi = files_list[z];
@@ -179,10 +213,11 @@ fsize_t vfu_update_sel_size( int one ) // used before copy/move to calc estimate
     if ( one && z != FLI ) continue; /* if one and not current -- skip */
     if ( !one && !fi->sel ) continue; /* if not one and not selected -- skip */
     if ( fi->is_link() ) continue; /* links does not have own size -- skip */
-    
+
     if ( fi->is_dir() )
       { /* this is directory */
-      fsize_t dir_size = vfu_dir_size( fi->name() );
+      fsize_t dir_size = vfu_dir_size( fi->name(), 0 );
+      need_size_cache_sort = 1;
       if ( dir_size == -1 )
         { /* dir size calculation has been canceled */
         size = -1;
@@ -196,6 +231,7 @@ fsize_t vfu_update_sel_size( int one ) // used before copy/move to calc estimate
       size += fi->size();
       }
     } /* for */
+  if( need_size_cache_sort ) size_cache_sort();
   update_status(); /*  */
   vfu_redraw(); /* just to redraw before copy/move/etc... */
   vfu_redraw_status(); /* just to redraw before copy/move/etc... */
@@ -224,7 +260,7 @@ VString& vfu_expand_mask( VString& mask )
 }
 
 /*---------------------------------------------------------------------------*/
-  
+
 char* time_str_compact( const time_t tim, char* buf )
 {
   ASSERT( buf );
@@ -352,7 +388,7 @@ static char hist_menu_hotkeys[] = "1234567890ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 int vfu_hist_menu( int x, int y, const char* title, int hist_id )
 {
   VString str;
-  
+
   mb.undef();
   int z;
   int cnt = vfu_hist_count( hist_id );
@@ -391,30 +427,30 @@ int vfu_get_str( const char *prompt, VString& target, int hist_id, int x, int y
   if ( x == -1 ) x = 1;
   if ( y == -1 ) y = con_max_y();
   int len = con_max_x() - 3 - x;
-  
+
   /* FIXME: this is not correct if x and y are specified */
   if ( prompt && prompt[0] )
     say1( prompt );
   say2( "" );
-  
+
   __vfu_get_str_hist_id = hist_id;
   if ( strcmp( target, "" ) == 0 && vfu_hist_get( hist_id, 0 ) )
     target = vfu_hist_get( hist_id, 0 );
   char t[1024]; /* FIXME: can be overflowed? */
   strcpy( t, target );
   int r = TextInput( x, y, "", len, len, t, vfu_get_str_history );
-  target = t; 
+  target = t;
   say1( "" );
   say2( "" );
   __vfu_get_str_hist_id = 0;
   if( r )
     vfu_hist_add( hist_id, target );
-  return ( r != 0 );  
+  return ( r != 0 );
 };
 
 /*---------------------------------------------------------------------------*/
 
-/* to fool gcc warning that mktemp() is not safe, I don't really care :/ 
+/* to fool gcc warning that mktemp() is not safe, I don't really care :/
    still there is no mkstemp for directories :)) */
 char vfu_temp_filename[MAX_PATH];
 const char* vfu_temp()
